open Atom
open Types

type coercion = 
  | CoId
      (* id *)
  | CoTrans of loc_coercion * loc_coercion 
      (* c1; c2 *)
  | CoIntroForall
      (* +forall *)
  | CoElimExists
      (* -exists *)
  | CoElimForall of bool *ftype
      (* -forall> t | -forall< t *)
  | CoIntroExists of bool *ftype
      (* +exists> t | +exists< t *)
  | CoArrow of loc_coercion * loc_coercion
      (* c1 -> c2 *)
  | CoQuantifier of quantifier * atom * loc_coercion
      (* all a. c *)
  | CoDistrib 
      (* distrib *)
  | CoDistribLabel of atom
      (* distrib l *)
  | CoRecord of loc_coercion AtomMap.t
      (* { l : c ) *)

and loc_coercion =
    Error.location * coercion

(* Determining a coercion's free term variables. *)

let rec occurs a c =
  match c with
  | CoId | CoIntroForall | CoElimExists | CoDistrib | CoDistribLabel _ -> 
      false
  | CoElimForall (_, t) | CoIntroExists (_, t) ->
      Types.occurs a t
  | CoQuantifier (q, a', c) -> 
      not (Atom.equal a a') && occurs a (unloc c)
  | CoArrow (c1, c2) | CoTrans (c1, c2) -> 
      occurs a (unloc c1) || occurs a (unloc c2)
  | CoRecord fields ->
      AtomMap.fold (fun label co res -> res || occurs a (unloc co))
        fields false
        

and unloc (_, c) = c



